#!/usr/bin/python
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon - DataBase
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################

#
# Logging
#
import logging
logger = logging.getLogger('tablon')

# Imports
import sqlite3
import threading
import time
import functools
import os
import utils
from distutils.version import LooseVersion as Version


#
# Reference to this module
#
import sys
tablondb = sys.modules[__name__]
tablondb.initiated = False

#
# The database connections
#
connections = dict()

#
# Create the connection for the current thread
#
def get_connection():

	thread = threading.current_thread()
	ident = thread.ident

	if not ident in connections:
		connections[ident] = sqlite3.connect('serverdata/tablondb.sqlite')
		connections[ident].row_factory = sqlite3.Row

	return connections[ident]



#
# Close all the database connections
# TODO: bug other threads can't close this
#
def close():

	return

	logger.info("Closing database connections %d" % len(connections) )

	for ident in connections:
		connections[ident].close()






#
# This is a decorator class
# This replaces the functions that use the dabase to obtain a DB cursor
# and do an optional commit at the end
# It also wrappers the execution in a try.
# The default value es returned if there is any exception
#
class dbcursor(object):

	# Decorator constructor
	def __init__(self,commit=False, default=None):

		# Commit flag and default value
		self.commit = commit
		self.default = default

	# Call the decorator
	def __call__(self, func=None):

		# If we have no function, this means the user has not properly
		# used the dbcursor decorator
		if func == None:
			raise Exception("You forgot the () in the decorator @dbcursor(), you need to init and call it!!! ")		

		# Reference to the decorator
		decorator = self

		# The returned function (javascript style)
		# This other decorator is used to mark the function
		# as a wraper for the debuger and trace
		@functools.wraps(func)
		def dbcursor_and_call(*args, **kwargs):

			# If it is not initiated and it isn't a init call
			# it can be an error if we do not define a default value
			if not tablondb.initiated and func.__name__ != 'init':
				if decorator.default != None:
					return decorator.default
				raise Exception("TablondB has not been initiated func '%s'" % func.__name__)


			# All is in a big try
			try:
				docommit = False

				# Connect and get cursor
				if not 'cursor' in kwargs:
					DB = get_connection()
					cursor = DB.cursor()
					docommit = True
				else:
					cursor = kwargs['cursor']
					del kwargs['cursor']

				# Call the orginal function
				res = func(cursor, *args, **kwargs)

				# Save the database
				if decorator.commit and docommit:
					DB.commit()

				# Return value
				return res

			except Exception as e:

				if decorator.default:
					return decorator.default

				logger.error("Error in database. Call to " + func.__name__)


				if isinstance(e,sqlite3.Error):
					print "sqlite3.Error", e.args[0]


				print e
				import traceback
				print traceback.format_exc()

				# Save the database
				#if decorator.commit and docommit:
				#	logger.error("Doing a rollback")
				#	DB.rollback()

				raise Exception("Database exception")


		return dbcursor_and_call




#
# Return the db version or None if it is empty
#
@dbcursor()
def get_dbversion(cursor):

	# Return the database version for 5.4+
	try:
		cursor.execute('SELECT version FROM tablondb')
		return Version(cursor.fetchone()[0])
	except:
		pass

	# The following code is to check old versions

	# Check if the database is empty
	cursor.execute('SELECT COUNT(*) FROM sqlite_master WHERE type="table"')
	if int(cursor.fetchone()[0]) == 0:
		return None

	# Check if we are 5.2 or 5.3
	try:
		cursor.execute('SELECT intxt FROM leaderboards')
		return Version('5.3')
	except:
		return Version('5.2')


	raise Exception("Version of database undefined")


@dbcursor()
def get_last_request(cursor):

	try:
		cursor.execute('SELECT MAX(rid) FROM requests')
		return int(cursor.fetchone()[0])
	except:
		return 0



#
# Init the database and create the tables if needed
#
@dbcursor(commit=True)
def init(cursor,userlist,leaderboards):

	logger.info("Init database ...")
	tablondb.initiated = True


	# Check version
	# ----------------------------------------------- #
	dbversion = get_dbversion(cursor=cursor)
	logger.info("Database file version: %s" % dbversion)


	# Add new version
	# ----------------------------------------------- #

	# TODO
	# Create the table	
	cursor.execute('CREATE TABLE IF NOT EXISTS tablondb(id INTEGER PRIMARY KEY, version TEXT)')
	cursor.execute('INSERT OR REPLACE INTO tablondb VALUES (1,"5.4")')



	# User table
	# ----------------------------------------------- #

	# Create the table	
	cursor.execute('''
		CREATE TABLE IF NOT EXISTS users(
			name TEXT PRIMARY KEY, quota INTEGER, usedquota FLOAT DEFAULT 0,
			requests INTEGER DEFAULT 0, errors INTEGER DEFAULT 0, admin INTEGER DEFAULT 0)
	''')

	# Remove db user that do not exists on the list
	cursor.execute('SELECT name FROM users')
	rows = cursor.fetchall()
	toremove = []
	for row in rows:
		name = row['name']
		if(not userlist.get(name)):
			toremove.append(name)

	# Remove the users
	for name in toremove:
		logger.debug('Removing user %s from db' % name)
		cursor.execute('DELETE FROM users WHERE name = ?',(name,))
			
	# Add the new users
	for user in userlist:
		cursor.execute('INSERT OR REPLACE INTO users(name, quota, admin) VALUES(?, ?, ?)',
			(user.name,user.quota,user.admin))
		update_user_stats(user,cursor=cursor)


	# Statuses table
	# ----------------------------------------------- #

	# Create the table	
	cursor.execute('''
		CREATE TABLE IF NOT EXISTS statuses
			(id INTEGER PRIMARY KEY, enum TEXT)
	''')

	values = [(1,"queued"),(2,"compiling"),(3,"deploying"),
			(4,"waiting"),(5,"running"),(6,"finished"),
			(7,"canceled"),(9,"error")]
	cursor.executemany('INSERT OR IGNORE INTO statuses VALUES(?,?)',values)


	# Request table
	# ----------------------------------------------- #

	# This is to update tablon 5.3 to 5.4
	# This can be removed in fresh instalations
	if dbversion and dbversion < Version('5.4'):
		logger.info("Updating requests table to tablondb 5.4")
		cursor.execute('ALTER TABLE requests ADD COLUMN hash TEXT')
		cursor.execute('ALTER TABLE requests ADD COLUMN lines INT')
		cursor.execute('ALTER TABLE requests ADD COLUMN tokens INT')
		cursor.execute('ALTER TABLE requests ADD COLUMN complexity INT')
		process_old_usercodes(cursor=cursor)

	# Create the table
	cursor.execute('''
		CREATE TABLE IF NOT EXISTS requests(rid INTEGER PRIMARY KEY,
		user TEXT REFERENCES users(name), program TEXT, arguments TEXT,
		queue TEXT, nprocs INT, threads INT, time REAL, leaderboard INT,
		output TEXT, status INT REFERENCES statuses(id), message TEXT,
		timestamp INT, address TEXT, addressname TEXT,
		hash TEXT, lines INT, tokens INT, complexity INT)
	''')



	# Not FINISHED -> ERROR
	cursor.execute('UPDATE requests SET status=9, message="Server crash" WHERE status<6')


	# Leaderboards table
	# ----------------------------------------------- #


	# This is to update tablon 5.2 to 5.3
	# This can be removed in fresh instalations
	if dbversion and dbversion < Version('5.3'):
		logger.info("Updating leaderboards table to tablondb 5.3")
		cursor.execute('ALTER TABLE leaderboards ADD COLUMN intxt INT')
	
	cursor.execute('''
		CREATE TABLE IF NOT EXISTS leaderboards(rid INTEGER PRIMARY KEY,
		user TEXT REFERENCES users(name), program TEXT,
		queue TEXT, time REAL, rtime REAL, timestamp INT, intxt INT)
	''')

	cursor.execute('''
		CREATE TABLE IF NOT EXISTS leaderboardsbackup(rid INTEGER PRIMARY KEY,
		user TEXT REFERENCES users(name), program TEXT,
		queue TEXT, time REAL, rtime REAL, timestamp INT, intxt INT)
	''')

	# Mark all the items as NOT in the txt
	cursor.execute('UPDATE leaderboards SET intxt=0')

	# Add leaderboards from files
	for lb in leaderboards:
		for item in lb:

			# Using the magic of the decorator we can pass
			# our current cursor
			# We update the item in the table and set the intxt flag
			add_leaderboard(item,cursor=cursor)

	# Make a backup and remove the items that are no in the txt
	cursor.execute('INSERT OR REPLACE INTO leaderboardsbackup SELECT * FROM leaderboards WHERE intxt=0')
	cursor.execute('DELETE FROM leaderboards WHERE intxt=0')




	# TODO : INDEX ¿?
	# ----------------------------------------------- #



	logger.info("Database DONE")




@dbcursor()
def process_old_usercodes(cursor):

	logger.info("Processing old usercodes")

	cursor.execute('SELECT * FROM requests')
	requests = cursor.fetchall()

	for request in requests:
		rid = int(request['rid'])
		ext = os.path.splitext(request['program'])[1]
		path = 'usercodes/src/'
		fname = "%sp%06d%s" % (path,rid,ext)
		fhash = utils.code_hash_file(fname)
		nloc, ccn, ntok = utils.code_stats_file(fname)
		cursor.execute('UPDATE requests SET hash = ?, lines = ?, tokens = ?, complexity = ? WHERE rid = ?',(fhash,nloc,ntok,ccn,rid))	



#
# Used quota of an user
#
@dbcursor(default=0)
def usedquota(cursor,name):

	# Get the quota
	cursor.execute('SELECT usedquota FROM users WHERE name = ?',(name,))
	row = cursor.fetchone()
	return int(row['usedquota'])



#
# Timestamp of the last request of one user
#
@dbcursor(default=0)
def last_timestamp(cursor,name):

	# Get the quota
	cursor.execute('SELECT  COALESCE(MAX(timestamp),0) AS timestamp FROM requests WHERE user = ?',(name,))
	row = cursor.fetchone()
	return int(row['timestamp'])



#
# Update the user stats: quota and errors
# We can pass a cursor to this function to speedup it when
# it is called in the init
#
@dbcursor(commit=True)
def update_user_stats(cursor,user):

	# Insert the request
	cursor.execute('UPDATE users SET usedquota=?, requests=?, errors=? WHERE name=?',
		(user.usedquota,user.requests,user.errors,user.name))






# Update the status, message, time, and output
@dbcursor(commit=True)
def update_request(cursor,r):

	# Insert the request
	values = (r.time,r.status,r.message,r.output,r.rid)
	cursor.execute('UPDATE requests SET time=?, status=?, message=?, output=? WHERE rid=?',values)




# Insert a new request
@dbcursor(commit=True)
def insert_request(cursor,r):

	# Insert the request
	values = (r.rid,r.user,r.program,r.arguments,r.queue,r.nprocs,r.threads,r.time,
	int(r.leaderboard),r.output,r.status,r.message,r.timestamp,
	r.address,r.addressname,
	r.hash,r.lines,r.tokens,r.complexity)
	cursor.execute('INSERT INTO requests VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?);',values)



@dbcursor()
def show_request_list(cursor,last=0):

	cursor.execute('''
		SELECT	rid,user,queue,nprocs,threads,program,status,
				message,time,leaderboard,statuses.enum AS statusname,timestamp
		FROM requests
		INNER JOIN statuses ON
			requests.status = statuses.id
		WHERE rid > ?
		ORDER BY rid DESC
		LIMIT ?
	''',(last,CONFIG.requestnumber))

	return cursor.fetchall()



@dbcursor()
def show_request(cursor,rid):
	
	cursor.execute('''
		SELECT	r.*, s.enum AS statusname, l.time AS ltime
		FROM requests r
		LEFT OUTER  JOIN leaderboards l ON
			r.rid = l.rid
		INNER  JOIN statuses s ON
			r.status = s.id
		WHERE r.rid = ?
	''',(rid,))

	return  cursor.fetchone()



@dbcursor(commit=True)
def add_leaderboard(cursor,item):

	# Insert the new item
	cursor.execute('''
		INSERT OR REPLACE INTO leaderboards VALUES(?,?,?,?,?,?,?,1)
	''',(item.rid,item.user,item.program,item.queue,item.time,item.rtime,item.timestamp))




@dbcursor()
def show_leaderboard(cursor,name):
	

	# V3: good performance using a temp table and ROWID
	# Speedup from 1.86s to 0.11s

	# Delete the temp table if exists
	cursor.execute('''
		DROP TABLE IF EXISTS templeaderboard;
	''')

	# Create a new table with this leaderboard,
	# insert first the ordered users and at the end the reference,
	# This way we can use the ROWID as pos.
	if None: # This uses a subquery but it is too slow
		cursor.execute('''
			CREATE TEMP TABLE templeaderboard as 
			SELECT lb.*
			FROM leaderboards lb
			WHERE lb.queue = ? AND lb.time = (
				SELECT MIN(time) FROM leaderboards lb2
				WHERE lb2.queue = ?  AND lb2.user = lb.user
			)
			ORDER BY lb.user <> 'reference' DESC, lb.time;
		''',(name,name))

	# This uses a join and is super fast :)
	cursor.execute('''
		CREATE TEMP TABLE templeaderboard as 
		SELECT lb2.*
		FROM(
			SELECT user, MIN(time) as mintime
			FROM leaderboards
			WHERE queue = ?
			GROUP BY user
		) lb1
		INNER JOIN leaderboards lb2 
		ON lb1.user = lb2.user AND lb1.mintime = lb2.time
		ORDER BY lb2.user <> 'reference' DESC, lb2.time;
	''',(name,))













	# Return the table and do a final order
	cursor.execute('SELECT ROWID AS pos, * FROM templeaderboard ORDER BY time')


	return cursor.fetchall()





@dbcursor()
def show_leaderboard_admin(cursor,name):

	# Delete the temp table if exists
	cursor.execute('DROP TABLE IF EXISTS templeaderboard2')

	# Create a new table with this leaderboard,
	# insert first the ordered users.
	# This way we can use the ROWID as pos.

	if None: # This uses a subquery but it is too slow
		cursor.execute('''
			CREATE TEMP TABLE templeaderboard2 as 
			SELECT lb.*
			FROM leaderboards lb
			WHERE lb.queue = ? AND lb.time = (
				SELECT MIN(time) FROM leaderboards lb2
				WHERE lb2.queue = ? AND lb.user <> 'reference' AND lb2.user = lb.user
			)
			ORDER BY lb.time;
		''',(name,name))


	# This uses a join and is super fast :)
	cursor.execute('''
		CREATE TEMP TABLE templeaderboard2 as 
		SELECT lb2.*
		FROM(
			SELECT user, MIN(time) as mintime
			FROM leaderboards
			WHERE queue = ? AND user <> 'reference' 
			GROUP BY user
		) lb1
		INNER JOIN leaderboards lb2 
		ON lb1.user = lb2.user AND lb1.mintime = lb2.time
		ORDER BY lb2.time;
	''',(name,))




	cursor.execute('''
		SELECT name as user2, tlb.*, tlb.ROWID AS pos
		FROM users u
		LEFT OUTER JOIN templeaderboard2 tlb
		ON u.name = tlb.user
		WHERE admin = 0
	''')


	return cursor.fetchall()





@dbcursor()
def show_leaderboard_admin_user(cursor,name,user):
	

	if None:
		cursor.execute('''
			SELECT *
			FROM leaderboards
			WHERE queue = ? AND user = ?
			ORDER BY time''',(name,user))

	cursor.execute('''
		SELECT r2.firstrid AS same, l.*, r.hash
		FROM (
			SELECT * FROM leaderboards
			WHERE queue = ? AND user = ?
			ORDER BY time
		) l
		INNER JOIN requests r ON r.rid = l.rid
		LEFT JOIN (
			SELECT MIN(rid) as firstrid, hash from requests
			GROUP BY hash
		) r2
		ON r.hash = r2.hash''',(name,user))




	return cursor.fetchall()







@dbcursor()
def show_users(cursor):
	
	cursor.execute('SELECT * FROM users ORDER BY admin, name',)

	return cursor.fetchall()



@dbcursor(default=None)
def get_user_from_ip(cursor,address,seconds=60*15):


	cursor.execute("""
		SELECT user
		FROM requests
		WHERE address=? AND timestamp > strftime('%s', 'now') - ?
		GROUP BY user ORDER BY timestamp DESC
		LIMIT 1""",(address,seconds))

	row = cursor.fetchone()
	if row != None:
		return row['user']
	else:
		return None
















# Functions for stats
# - # - # - # - # - # - # - # - # - # - # - # - # - # - # - # - # - # - # - # 






@dbcursor()
def show_stats_request_status(cursor,ini,end):
	
	cursor.execute('''
		SELECT statuses.enum AS name, COUNT(*) AS number
		FROM requests
		INNER JOIN statuses ON requests.status = statuses.id
		WHERE timestamp > ? AND timestamp < ?
		GROUP BY status
	''',(ini,end))

	return cursor.fetchall()

@dbcursor()
def show_stats_error_types(cursor,ini,end):
	
	cursor.execute('''
		SELECT COUNT(rid) AS number, message
		FROM requests
		WHERE status=9 AND timestamp > ? AND timestamp < ?
		GROUP BY message
	''',(ini,end))

	return cursor.fetchall()







# Number of days
# SELECT COUNT(DISTINCT date(timestamp, 'unixepoch')) FROM requests 


# SELECT date(MIN(timestamp), 'unixepoch') AS min, date(MAX(timestamp), 'unixepoch') AS max FROM requests


@dbcursor()
def show_stats_total_requests(cursor,ini,end):

	cursor.execute('''
		SELECT COUNT(*) AS number, COALESCE(SUM(time),0) AS time
		FROM requests
		WHERE timestamp > ? AND timestamp < ?
	''',(ini,end))

	return cursor.fetchone()


@dbcursor()
def show_stats_first_after_ts(cursor,ts):

	cursor.execute('''
		SELECT MIN(timestamp) AS first
		FROM requests
		WHERE timestamp > ?
	''',(ts,))

	return cursor.fetchone()


@dbcursor()
def show_stats_last_before_ts(cursor,ts):

	cursor.execute('''
		SELECT MAX(timestamp) AS first
		FROM requests
		WHERE timestamp < ?
	''',(ts,))

	return cursor.fetchone()



# Number of requests each day
@dbcursor()
def show_stats_request_byday(cursor,ini,end):

	cursor.execute('''
		SELECT date(timestamp, 'unixepoch') AS date, COUNT(*) AS number
		FROM requests
		WHERE timestamp > ? AND timestamp < ?
		GROUP BY date(timestamp, 'unixepoch')
	''',(ini,end))

	return cursor.fetchall()



@dbcursor()
def show_stats_request_byhour(cursor,ini,end):

	#cursor.execute('''
	#	SELECT strftime('%H', timestamp, 'unixepoch') AS hour, COUNT(*) AS number
	#	FROM requests
	#	GROUP BY strftime('%H',timestamp, 'unixepoch')
	#	ORDER BY hour''')

	cursor.execute('''

		SELECT hour, MAX(number) as number
		FROM (
			SELECT strftime('%H', timestamp, 'unixepoch') AS hour, COUNT(*) AS number
					FROM requests
					WHERE timestamp > ? AND timestamp < ?
					GROUP BY strftime('%H',timestamp, 'unixepoch')
			UNION
			SELECT '00' as hour, 0 as number
			UNION
			SELECT '01' as hour, 0 as number
			UNION
			SELECT '02' as hour, 0 as number
			UNION
			SELECT '03' as hour, 0 as number
			UNION
			SELECT '04' as hour, 0 as number
			UNION
			SELECT '05' as hour, 0 as number
			UNION
			SELECT '06' as hour, 0 as number
			UNION
			SELECT '07' as hour, 0 as number
			UNION
			SELECT '08' as hour, 0 as number
			UNION
			SELECT '09' as hour, 0 as number
			UNION
			SELECT '10' as hour, 0 as number
			UNION
			SELECT '11' as hour, 0 as number
			UNION
			SELECT '12' as hour, 0 as number
			UNION
			SELECT '13' as hour, 0 as number
			UNION
			SELECT '14' as hour, 0 as number
			UNION
			SELECT '15' as hour, 0 as number
			UNION
			SELECT '16' as hour, 0 as number
			UNION
			SELECT '17' as hour, 0 as number
			UNION
			SELECT '18' as hour, 0 as number
			UNION
			SELECT '18' as hour, 0 as number
			UNION
			SELECT '19' as hour, 0 as number
			UNION
			SELECT '20' as hour, 0 as number
			UNION
			SELECT '21' as hour, 0 as number
			UNION
			SELECT '22' as hour, 0 as number
			UNION
			SELECT '23' as hour, 0 as number
		)
		GROUP BY hour
		ORDER BY hour

	''',(ini,end))


	return cursor.fetchall()



@dbcursor()
def show_stats_request_byweekday(cursor,ini,end):

	cursor.execute('''

		SELECT day, MAX(number) as number
		FROM (
			SELECT strftime('%w', timestamp, 'unixepoch') AS day, COUNT(*) AS number
					FROM requests
					WHERE timestamp > ? AND timestamp < ?
					GROUP BY strftime('%w',timestamp, 'unixepoch')
			UNION
			SELECT '0' as day, 0 as number
			UNION
			SELECT '1' as day, 0 as number
			UNION
			SELECT '2' as day, 0 as number
			UNION
			SELECT '3' as day, 0 as number
			UNION
			SELECT '4' as day, 0 as number
			UNION
			SELECT '5' as day, 0 as number
			UNION
			SELECT '6' as day, 0 as number
		)
		GROUP BY day
		ORDER BY day

	''',(ini,end))


	return cursor.fetchall()





@dbcursor()
def show_stats_leaderboard_plot(cursor,lbname):

	"""
	This is the big function to create the leaderboard plot
	"""

	import datetime


	# Get the min timestamp
	cursor.execute('''	SELECT min(timestamp) FROM leaderboards
						WHERE queue = ?''',(lbname,))
	mintimestamp = cursor.fetchone()[0]


	# Get the max timestamp
	cursor.execute('''SELECT	max(timestamp) FROM leaderboards
								WHERE queue = ?''',(lbname,))
	maxtimestamp = cursor.fetchone()[0]


	# Seconds in a day
	dayseconds = 86400


	# Get the min and max days
	mintimestamp = int((mintimestamp/dayseconds)*dayseconds)
	maxtimestamp = int((maxtimestamp/dayseconds)*dayseconds)


	# Create the temp table
	cursor.execute('DROP TABLE IF EXISTS temp_dominating1')
	cursor.execute('CREATE TEMP TABLE temp_dominating1 (user TEXT, pos INT, dayts INT)')


	# Iterate for each day
	for ts in range(mintimestamp,maxtimestamp+dayseconds,dayseconds):

		# Delete the second temp table
		cursor.execute('DROP TABLE IF EXISTS temp_dominating2')

		# Create a new table with this leaderboard,
		# insert first the ordered users.
		# This way we can use the ROWID as pos.
		if None:
			cursor.execute('''
				CREATE TEMP TABLE temp_dominating2 as 
				SELECT lb.*
				FROM leaderboards lb
				WHERE lb.queue = ? AND lb.time = (
					SELECT MIN(time) FROM leaderboards lb2
					WHERE lb2.queue = ? AND lb.user <> 'reference'
					AND lb2.user = lb.user AND lb2.timestamp < ?
				)
				ORDER BY lb.time;
			''',(lbname,lbname,ts+dayseconds))


		# This uses a join and is super fast :)
		cursor.execute('''
			CREATE TEMP TABLE temp_dominating2 as 
			SELECT lb2.*
			FROM(
				SELECT user, MIN(time) as mintime
				FROM leaderboards
				WHERE queue = ? AND user <> 'reference' AND timestamp < ? 
				GROUP BY user
			) lb1
			INNER JOIN leaderboards lb2 
			ON lb1.user = lb2.user AND lb1.mintime = lb2.time
			ORDER BY lb2.time;
		''',(lbname,ts+dayseconds))





		# Add temp2 into temp1 with the rowid as pos
		cursor.execute('''
			INSERT INTO temp_dominating1
			SELECT user, ROWID, ?
			FROM temp_dominating2
		''',(ts,))



	# Get the max pos, total number of tries
	cursor.execute('SELECT max(pos) FROM temp_dominating1')
	maxpos = int(cursor.fetchone()[0])


	# Get the list of users
	cursor.execute('SELECT user FROM temp_dominating1 GROUP BY user')
	rows = cursor.fetchall()

	users = []
	for row in rows:
		users.append(row[0])



	# Result
	# The result is dict with users as keys
	# each user has a list with tuples (day, pos)
	result = {}


	# Iterate for the users
	for user in users:

		# Get the dates for one user
		cursor.execute('SELECT * FROM temp_dominating1 WHERE user = ?',(user,))	
		rows = cursor.fetchall()

		# Compose the result for the user
		result[user] = []
		for row in rows:
			date = datetime.datetime.fromtimestamp(row['dayts'])
			pos = int(row['pos'])
			result[user].append((date,pos))


	# Return the result
	return result






#def request_get_range_dates():


#SELECT date(MIN(timestamp), 'unixepoch') AS min, date(MAX(timestamp), 'unixepoch') AS max FROM requests

# SELECT strftime('%H', timestamp, 'unixepoch') as hour, COUNT(*) as number FROM requests GROUP BY strftime('%H', timestamp, 'unixepoch')

# SELECT count(distinct user) AS shared, hash FROM requests GROUP BY hash ORDER BY  shared DESC
# 3757

# SELECT user, program, count(user) AS number, count(distinct user) AS shared, hash FROM requests GROUP BY hash HAVING shared < 10 AND number > 1  ORDER BY  number DESC




